#!/usr/bin/python
# -*- coding: iso-8859-7 -*-
import random, copy
from math import cos, sin, pi, hypot, fabs
from p_ggen import prg, iterby2, xfrange, xfrangec, Struct
from p_gmath import thanLineSeg3, thanNear2
from p_gtri import ThanDTMlines, hull
from p_gchart import ThanChart, vis


def itercom(fr):
        "Iterates through a file ignoring comments."
        for dl in fr:
            dl = dl.strip()
            if dl == "" or dl[0] == "#": continue
            yield dl


class RoadGrid:
    "An object which contains a grid of road with nonuniform grid spaces."

    def __init__(self, hull, dtm):
        "Initialize a road grid."
        self.hull = hull
        self.dtm = dtm
        self.roads = []


    def build_cache(self):
        "Create road grid every dx, dy, dtheta; cache the results."
        self.dx = self.dy = 1.0
        self.dtheta = 1.0
        self.roadenx = {}
        self.roadeny = {}
        self.pxmin = {}
        self.pxmax = {}
        self.pymin = {}
        self.pymax = {}
        print "Building cache.."
        for theta in xfrange(-90.0, 90.0, self.dtheta):
            print "theta=", theta
            thrad = theta*pi/180.0
            pymin, pymax = self.cache_roads(self.roadenx, theta, thrad, self.dy)
            pxmin, pxmax = self.cache_roads(self.roadeny, theta, thrad-pi*0.5, self.dx)
            self.pymin[theta] = pymin
            self.pymax[theta] = pymax
            self.pxmin[theta] = pxmin
            self.pxmax[theta] = pxmax


    def cache_roads(self, roads, theta, thrad, dy):
        "Create grid; direction theta with respect to x-axis, varied widths,lenths of O.T. varies lengths of roads."
        t = cos(thrad), sin(thrad)
        n = -t[1], t[0]
        py = [n[0]*c[0]+n[1]*c[1] for c in self.hull]
        pymin = int(min(py)/dy+1.1)*dy     # Avoid py exactly on boundary
        pymax = int(max(py)/dy-1.1)*dy     # Avoid py exactly on boundary

        for pyaxis in xfrangec(pymin, pymax, dy):
            c1, c2 = thanPolygonLine(self.hull, pyaxis, n)
            assert c1 != None, "There should be 2 intersections!"
            nc, cprof = self.dtm.thanLineZ((c1, c2))
            if nc == -1:
                roads[theta, pyaxis] = 0.0, 0.0      # No profile was found
            else:
                roads[theta, pyaxis] = self.cache_roadenergy(cprof)
        return pymin, pymax


    def write_cache(self, fw):
        "Write the cache to a file."
        fw.write("# dx dy dtheta:\n%f   %f   %f\n" % (self.dx, self.dy, self.dtheta))
        for nam, pc in ("pxmin", self.pxmin), ("pxmax", self.pxmax), ("pymin", self.pymin), ("pymax", self.pymax):
            fw.write("\n# %s:\n" % nam)
            for key in sorted(pc.iterkeys()):
                fw.write("%f   %f\n" % (key, pc[key]))
            fw.write("$\n")
        for nam, pc in ("road energy x", self.roadenx), ("road energy y", self.roadeny):
            fw.write("\n# %s:\n" % nam)
            for key in sorted(pc.iterkeys()):
                val = pc[key]
                fw.write("%f  %f   %f  %f  %f  %f\n" % (key[0], key[1], val[0], val[1], val[2], val[3]))
            fw.write("$\n")


    def read_cache(self, fr):
        "Write the cache to a file."
        self.dx = self.dy = 1.0
        self.dtheta = 1.0
        self.roadenx = {}
        self.roadeny = {}
        self.pxmin = {}
        self.pxmax = {}
        self.pymin = {}
        self.pymax = {}
        it = itercom(fr)

        self.dx, self.dy, self.dtheta = map(float, it.next().split())

        for nam, pc in ("pxmin", self.pxmin), ("pxmax", self.pxmax), ("pymin", self.pymin), ("pymax", self.pymax):
            while True:
                dline = it.next()
                if dline == "$": break
                key, val = map(float, dline.split())
                pc[key] = val

        for nam, pc in ("road energy x", self.roadenx), ("road energy y", self.roadeny):
            while True:
                dline = it.next()
                if dline == "$": break
                key1, key2, val1, val2, val3, val4 = map(float, dline.split())
                pc[key1, key2] = val1, val2, val3, val4



    w1 = 10.0
    w2 = 100.0
    def cache_roadenergy(self, cc):
        "Calculates the energy of a road."
        sd = e = 0.0
        d8 = 0.0
        d10 = 0.0
        for ca, cb in iterby2(cc):
                d = hypot(cb[0]-ca[0], cb[1]-ca[1])
                sd += d
                dz = fabs(cb[2]-ca[2])
                slope = dz*100.0/d
                if slope <= 8.0:
                    e += 0.0
                elif slope < 10.0:
                    e += d*self.w1*(slope-8.0)
                    d8 += d
                else:
                    e += d*self.w1*(10.0-8.0) + d*self.w2*(slope-10.0)
                    d8 += d
                    d10 += d
        return sd, d8, d10, e


    def iterOT(self, state):
        "Iterate through all OT."
        t = cos(state.theta*pi/180.0), sin(state.theta*pi/180.0)
        n = -t[1], t[0]
        px = [t[0]*c[0]+t[1]*c[1] for c in self.hull]
        py = [n[0]*c[0]+n[1]*c[1] for c in self.hull]
        pxmin = min(px)
        pxmax = max(px)
        pymin = min(py)
        pymax = max(py)

        print "bod=", state.bod
        print "boik=", state.boik
        print "hoik=", state.hoik

        pya = pymin
        jy = jjy = 0
        while True:
            pya += state.bod[jjy]
            pyb = pya+state.boik[jjy]
            if pyb > pymax: break
            ca1, ca2 = thanPolygonLine(self.hull, pya, n)
            assert ca1 != None, "There should be 2 intersections!"
            cb1, cb2 = thanPolygonLine(self.hull, pyb, n)
            assert cb1 != None, "There should be 2 intersections!"
            pxa1 = t[0]*ca1[0]+t[1]*ca1[1]
            pxa2 = t[0]*ca2[0]+t[1]*ca2[1]
            if pxa1 > pxa2: ca1, pxa1, ca2, pxa2 = ca2, pxa2, ca1, pxa1
            pxb1 = t[0]*cb1[0]+t[1]*cb1[1]
            pxb2 = t[0]*cb2[0]+t[1]*cb2[1]
            if pxb1 > pxb2: cb1, pxb1, cb2, pxb2 = cb2, pxb2, cb1, pxb1
            px1min = max((pxa1, pxb1))
            px2max = min((pxa2, pxb2))

            px1 = pxmin
            jx = jjx = 0
            while True:
                px1 += state.bod[jjx]
                px2 = px1 + state.hoik[jjx]
                if px2 > px2max: break
                if px1 >= px1min:
                    cc = []
                    for px, py in (px1, pya), (px2, pya), (px2, pyb), (px1, pyb), (px1, pya):
                        cc.append([px*t[0]+py*n[0], px*t[1]+py*n[1]])
                    yield cc
                px1 = px2
                jx += 1
                if jx < len(state.hoik): jjx = jx

            pya = pyb
            jy += 1
            if jy < len(state.boik): jjy = jy


    def energy(self, s):
        "Calculates the energy of the system or the difference to the ideal configuration."
        sd, sd8, sd10, se = self.roadenergy(self.roadenx, self.dy, s.theta, self.pymin, self.pymax, s.boik, s.bod)
        d, d8, d10, e     = self.roadenergy(self.roadeny, self.dx, s.theta, self.pxmin, self.pxmax, s.hoik, s.bod)
        sd += d
        sd8 += d8
        sd10 += d10
        se += e
        return se/sd


    def energy3(self, s):
        "Calculates the energy of the system or the difference to the ideal configuration."
        sd, sd8, sd10, se = self.roadenergy(self.roadenx, self.dy, s.theta, self.pymin, self.pymax, s.boik, s.bod)
        d, d8, d10, e     = self.roadenergy(self.roadeny, self.dx, s.theta, self.pxmin, self.pxmax, s.hoik, s.bod)
        sd += d
        sd8 += d8
        sd10 += d10
        se += e
        return se/sd, sd8, sd10


    def roadenergy(self, roaden, dy, theta, pymin1, pymax1, boik, bod):
        "Find the energy of roads along x or y axis."
        py = pymin1[theta]
        pymax = pymax1[theta]
        j = jj = 0
        sd = sd8 = sd10 = se = 0.0
        while True:
            pyaxis = py + bod[jj]*0.5
            pyaxis = int(pyaxis/dy+0.5)*dy
            if pyaxis > pymax: break
            d, d8, d10, e = roaden[theta, pyaxis]
            sd += d
            sd8 += d8
            sd10 += d10
            se += e
            py += bod[jj] + boik[jj]
            j += 1
            if j < len(boik): jj = j
        return sd, sd8, sd10, se


    def roadcoor(self, s):
        "Find road coordinates along x and y axis."
        thrad = s.theta*pi/180.0
        roads = []
        self.roadcoor1(roads, self.dy, s.theta, thrad,        self.pymin, self.pymax, s.boik, s.bod)
        self.roadcoor1(roads, self.dx, s.theta, thrad-pi*0.5, self.pxmin, self.pxmax, s.hoik, s.bod)
        return roads


    def roadcoor1(self, roads, dy, theta, thrad, pymin1, pymax1, boik, bod):
        "Find road coordinates along x or y axis."
        t = cos(thrad), sin(thrad)
        n = -t[1], t[0]
        py = pymin1[theta]
        pymax = pymax1[theta]
        j = jj = 0
        while True:
            pyaxis = py + bod[jj]*0.5
            pyaxis = int(pyaxis/dy+0.5)*dy
            if pyaxis > pymax: break
            c1, c2 = thanPolygonLine(self.hull, pyaxis, n)
            assert c1 != None, "There should be 2 intersections!"
            roads.append((c1, c2))
            py += bod[jj] + boik[jj]
            j += 1
            if j < len(boik): jj = j


    def tochart(self, ch, iso=None, ot=None, roads=None):
        "Show the road grid."
        xx = [c[0] for c in self.hull]
        yy = [c[1] for c in self.hull]
        ch.curveAdd(xx, yy, color="green")
        if roads != None:
            for cp in roads:
                xx = [c[0] for c in cp]
                yy = [c[1] for c in cp]
                ch.curveAdd(xx, yy, style="dasheddot", size=(12, -6, 3, -6), color="red")
        if iso != None:
            col = "brown"
            for cp in iso:
                xx = [c[0] for c in cp]
                yy = [c[1] for c in cp]
                ch.curveAdd(xx, yy, color=col)
        if ot != None:
            col = "blue"
            for cp in ot:
                xx = [c[0] for c in cp]
                yy = [c[1] for c in cp]
                ch.curveAdd(xx, yy, color=col)


    def todxf(self, dxf, iso=None, ot=None, roads=None):
        "Show the road grid."
        from p_gimdxf.thancolors import thanDxfColName2Rgb, thanRgb2DxfColCodeApprox
        xx = [c[0] for c in self.hull]
        yy = [c[1] for c in self.hull]
        _layer(dxf, "hull", "green")
        dxf.thanDxfPlotPolyline(xx, yy)
        if roads != None:
            _layer(dxf, "roads", "red")
            for cp in roads:
                xx = [c[0] for c in cp]
                yy = [c[1] for c in cp]
                dxf.thanDxfPlotPolyline(xx, yy)
        if iso != None:
            _layer(dxf, "contours", "brown")
            for cp in iso:
                xx = [c[0] for c in cp]
                yy = [c[1] for c in cp]
                dxf.thanDxfPlotPolyline(xx, yy)
        if ot != None:
            _layer(dxf, "OT", "blue")
            for cp in ot:
                xx = [c[0] for c in cp]
                yy = [c[1] for c in cp]
                dxf.thanDxfPlotPolyline(xx, yy)


    def show(self, title="", iso=None, ot=None, roads=None):
        "Show the road grid."
        ch = ThanChart(title)
        self.tochart(ch, iso, ot, roads)
        vis(ch, bg="white", title=title)


    def dxfout(self, title="", iso=None, ot=None, roads=None):
        "Plot the road grid to a dxf file."
        from p_gdxf import ThanDxfPlot
        dxf = ThanDxfPlot()
        dxf.thanDxfPlots()
        self.todxf(dxf, iso, ot, roads)
        dxf.thanDxfPlot(0,0,999)


def thanPolygonLine(cpol, pn, n):
    "Finds the intersection between convex polygon segment and line perpendicular to n, whose projection on n axis is pn."
    cts = []
    for i in xrange(len(cpol)):
        ct = thanLineSeg3(pn, n, cpol[i-1], cpol[i])
        if ct != None: cts.append(ct)
    if len(cts) == 0: return None, None
    cts.sort()
    j = 1
    while j < len(cts):
        i = j - 1
        if thanNear2(cts[j], cts[i]):
            del cts[j]
        else:
            j += 1
    assert len(cts) == 2, "There should be exactly 2 intersections"
    return cts[0], cts[1]


def lineprofile(clines):
    "Creates a (3D) line profile into a new drawing."
    dfact = 0.1
    charts = []
    for cp,pyaxis,col in clines:
        d = [0.0*dfact]
        for ca, cb in iterby2(cp):
            d.append(d[-1]+hypot(cb[1]-ca[1], cb[0]-ca[0])*dfact)
        zmin = min(ca[2] for ca in cp)
        ch = ThanChart()
        ch.curveAdd((d[0], d[-1]), (0.0, 0.0), color=col)
        ch.curveAdd((d[0], d[0]), (0.0, cp[0][2]-zmin))
        for i in xrange(1, len(cp)):
            ch.curveAdd((d[i], d[i], d[i-1]), (0.0, cp[i][2]-zmin, cp[i-1][2]-zmin), color=col)
        charts.append(ch)
    return charts


def _layer(dxf, lay, col):
    "Sets layer and color."
    from p_gimdxf.thancolors import thanDxfColName2Rgb, thanRgb2DxfColCodeApprox
    rgb = thanDxfColName2Rgb[col]
    icol = thanRgb2DxfColCodeApprox(rgb)
    dxf.thanDxfSetLayer(lay)
    dxf.thanDxfSetColor(icol)


class PolConfiguration:
    "A grid of roads in region amenable to simulated annealling."

    def __init__(self, hull, dtm):
        "Initial configuration is a single room."
        self.hull = hull
        self.dtm = dtm
        self.pol = RoadGrid(hull, dtm)

        self.boik1 = 8.0+15.0+8.0      # Perpendicular to ideal direction (building+free space+building)
        self.boik2 = 12.0+25.0+12.0
        self.hoik1 = 50.0              # Parallel to ideal direction
        self.hoik2 = 100.0
        self.bod1 = 2.0+10.0+2.0       # Road width (pavement+width+pavement)
        self.bod2 = 2.5+12.0+2.5

        self.boik1 = 25.0+25.0        # Perpendicular to ideal direction (property1+property2)
        self.boik2 = 30.0+30.0        # Perpendicular to ideal direction (property1+property2)
        self.hoik1 = 60.0             # Parallel to ideal direction
        self.hoik2 = 140.0 
        self.bod1 = 2.5+7.0+2.5       # Road width (pavement+width+pavement)
        self.bod2 = 2.5+8.0+2.5

        self.state = Struct()
        self.state.e = -1.0           # Invalid value for energy
        self.state.boik = [self.boik1]
        self.state.hoik = [self.hoik1]
        self.state.bod = [self.bod1]
        self.state.theta = 0.0
#        self.state.theta = 90.0
        self.ndim = 1+1

        self.r = random.Random()


    def setState(self):
        "Initalizes state after cache has been created or read."
        nb = int( (self.pol.pymax[0]-self.pol.pymin[0]) / (self.bod1+self.boik1) + 0.5)
        nh = int( (self.pol.pxmax[0]-self.pol.pxmin[0]) / (self.bod1+self.hoik1) + 0.5)
        print "nb=", nb, "nh=", nh
        self.state.boik = [self.boik1]*nb
        self.state.hoik = [self.hoik1]*nh
        self.state.bod = [self.bod1]*max(nb, nh)
        self.state.theta = 0.0
#        self.state.theta = 90.0
        self.ndim = 2*(nb + nh)


    def test(self):
        "Tests energy for various angles."
        for theta in xfrange(-90, 90, 10):
            self.state.theta = theta*pi/180.0
            self.pol.create_geometry(self.state)
            print "theta=%8.1f   energy=%15.6f" % (theta, self.pol.energy())


    def energyState(self):
        "Computes the energy of the configuration."
        return self.pol.energy(self.state) * self.efact


    def changeState(self):
        "Changes the configuration a little."
        self.saveOldState()
        p = self.r.random()
        if p < 0.20:      # Change theta
            theta = self.state.theta
#            if theta < 0.0: theta = self.state.theta = theta+180.0
            while True:
                dth = 5.0
                theta = self.state.theta + self.r.uniform(-dth, dth)
                theta = int(theta/self.pol.dtheta)*self.pol.dtheta
                if -90.0 <= theta < 90.0: break
#                if 90.0-22.5 <= theta < 90.0+22.5: break
#            if theta > 89.0: theta = theta-180.0
            self.state.theta = theta
        elif p < 0.4 :    # Change width of OT
            i = self.r.randint(0, len(self.state.boik)-1)
            self.state.boik[i] = int(self.r.uniform(self.boik1, self.boik2)/self.pol.dy)*self.pol.dy
        else:             # Change height of OT
            i = self.r.randint(0, len(self.state.hoik)-1)
            self.state.hoik[i] = int(self.r.uniform(self.hoik1, self.hoik2)/self.pol.dx)*self.pol.dx
        return self.ndim  # Dimensionality of the configuration


    def saveOldState(self):
        "Save current configuration for future unChangeState()."
        self.stateOld = copy.deepcopy(self.state)


    def unChangeState(self):
        "Restores previously saved state."
#       print "unchange"
        self.state = self.stateOld
#        self.typeNodes()
        return self.ndim       # Dimensionality of the configuration


    def saveMinState(self):
        "Save current configuration as the best so far."
        self.stateMin = copy.deepcopy(self.state)


    def restoreMinState(self):
        "Restores previously saved best configuration."
        self.state = self.stateMin
#        self.typeNodes()
        return self.ndim       # Dimensionality of the configuration


    def initState(self):
      "Initialize random changes and calibrate energy."
      self.efact = 1.0
      ndim = self.ndim
      tries = ndim*10
      for i in xrange(tries):        # Randomise a bit
          j = self.changeState()
      e1 = self.energyState()
      de = 0.0
      ndim = self.ndim
      tries = ndim*10
      for i in xrange(tries):
          j = self.changeState()
          e2 = self.energyState()
          de += abs(e2-e1)
          e1 = e2
      de /= tries
      self.efact = 100.0 / de             # Normalise delta energy to 100: efact*de = 100
      prg(' =%.3f -> %3.f' % (e1, e1*self.efact))
      prg("   =%.3f -> %3.f" % (de, de*self.efact))
      print "efact=", self.efact
#      self.plot("First approximation")
      return self.ndim


    def wrstate(self, fw):
        "Save the current state to a file."
        s = self.state
        e, d8, d10 = self.pol.energy3(s)
        fw.write("# Energy e, length with slope > 8%, length with slope > 10%:\n")
        fw.write("%f  %f  %f\n" % (e, d8, d10))
        fw.write("\n# theta:\n")
        fw.write("%f\n" % s.theta)
        for nam, b in ("boik", s.boik), ("hoik", s.hoik), ("bod", s.bod):
            fw.write("\n# %s:\n" % nam)
            for a in b: fw.write("%f\n" % a)
            fw.write("$\n")


    def rdstate(self, fr):
        "Reads the current state from a file."
        s = self.state
        it = itercom(fr)
        s.e, s.d8, s.d10 = map(float, it.next().split())
        s.theta = float(it.next())
        for nam, b in ("boik", s.boik), ("hoik", s.hoik), ("bod", s.bod):
            del b[:]
            while True:
                dline = it.next()
                if dline == "$": break
                b.append(float(dline))


if __name__ == "__main__":
    pass
