#!/usr/bin/python
from p_gtri import ThanDTMlines, hull
from anneal import SimulatedAnnealing
from road import PolConfiguration


def readsyk(fr):
    "Reads the contours of a syk file."
    lindxf = 0
    it = iter(fr)
    clines = []
    for dline in it:
        lindxf += 1
        try:
            z1 = float(dline[:15])
        except (ValueError, IndexError), why:
            why = "%s %d of file .syk:\n%s" % ("Error at line", lindxf, why)
            raise ValueError, why
        cc = []
        for dline in it:
            lindxf += 1
            if dline.strip() == "$": break
            try:
                x1 = float(dline[:15])
                y1 = float(dline[15:30])
            except (ValueError, IndexError), why:
                why = "%s %d of file .syk:\n%s" % ("Error at line", lindxf, why)
                raise ValueError, why
            else: cc.append([x1, y1, z1])

        if len(cc) < 2: print "Polyline with 1 or 0 vertices."
        clines.append(cc) #-----------Store the polyline
    return clines


def makeDTM(fn):
    "Makes DTM and hull from a .syk file."
    fr = open(fn)
    clines = readsyk(fr)
    fr.close()
    dtm = ThanDTMlines()
    for cline1 in clines: dtm.thanAddLine1(cline1)
    dtm.thanRecreate()
    cc = []
    for cline1 in clines: cc.extend(cline1)
    hu = hull(cc)
    return hu, dtm


def build_cache():
    "Build and writes cache."
    hu, dtm = makeDTM("kam5k.syk")
    pc = PolConfiguration(hu, dtm)
    pc.pol.build_cache()
    fw = open("kam5k_cache01.txt", "w")
    pc.pol.write_cache(fw)
    fw.close()


def test_cache():
    "Reads and writes cache for test."
    hu, dtm = makeDTM("kam5k.syk")
    pc = PolConfiguration(hu, dtm)
    fr = open("kam5k_cache01.txt", "r")
    pc.pol.read_cache(fr)
    fr.close()
    fw = open("kam5k_cache02.txt", "w")
    pc.pol.write_cache(fw)
    fw.close()


def show(fn):
    "Show the result."
    hu, dtm = makeDTM("kam5k.syk")
    pc = PolConfiguration(hu, dtm)
    try:
        pc.dx
    except:
        pass
#        fr = open("kam5k_cache01.txt", "r")
#        pc.pol.read_cache(fr)
#        fr.close()

    pc.rdstate(open(fn))
    s = pc.state
    ot = list(pc.pol.iterOT(s))
#    roads = pc.pol.roadcoor(s)
    roads = None
    tit = "Energy=%.1f,  d>8%%=%.1f,  d>10%%=%.1f" % (s.e, s.d8, s.d10)
    pc.pol.show(title=tit, iso=dtm.thanLines, ot=ot, roads=roads)
    pc.pol.dxfout(title=tit, iso=dtm.thanLines, ot=ot, roads=roads)


def thanMain():
    "Main routine."
    hu, dtm = makeDTM("kam5k.syk")
    pc = PolConfiguration(hu, dtm)
    fr = open("kam5k_cache01.txt", "r")
    pc.pol.read_cache(fr)
    fr.close()

    pc.setState()
    sa = SimulatedAnnealing()
    sa.anneal(pc)
    e = pc.energyState() / pc.efact
    print "final energy=", e, " ->", e*pc.efact
    fw = open("state01.txt", "w")
    pc.wrstate(fw)
    fw.close()

#    rc.show("Final configuration")

if __name__ == "__main__": 
#    build_cache()
#    test_cache()
#    thanMain()
    show("state01.txt")
