#!/usr/bin/python
# -*- coding: iso-8859-7 -*-
import random, copy
from p_ggen import prg

class Room:
    "A class which represents a room, posible divided."

    def __init__(self, xymm):
        "The xymm are the coordinate of lower left followe by the coordinate  of upper right."
        self.xymm = list(xymm)
        self.isplit = None         # Room is not (yet) split
        self.csplit = None
        self.roomll = self.roomur = None # Children

    def split(self, isplit, csplit):
        "Split the room to 2 children."
	assert self.isleaf(), "room is already split!"
	assert self.xymm[isplit] < csplit < self.xymm[isplit+2], "Split line outside of the room"
	self.isplit = isplit
	self.csplit = csplit
	xymm = list(self.xymm)
	xymm[isplit+2] = csplit
	self.roomll = Room(xymm)       
	xymm = list(self.xymm)
	xymm[isplit] = csplit
	self.roomur = Room(xymm)       

    def mergeold(self):
        "Merges the room's children."
	assert self.roomll.isleaf() and self.roomur.isleaf(), "Children are not single rooms"
	self.isplit = self.csplit = self.roomll = self.roomur = None

    def merge(self):
        "Merges all the children/grandchildren of the room."
	self.isplit = self.csplit = self.roomll = self.roomur = None

    def isleaf(self):
        "Checks if the room is a leaf node (it has not been split)."
	return self.isplit == None

    def list(self, all, mall, sall, chall):
        "Returns a list of self and children."
	all.append(self)
	if self.isleaf():
	    sall.append(self)
	else:
	    if self.roomll.isplit == None and \
	       self.roomur.isplit == None: chall.append(self)
	    mall.append(self)
	    self.roomll.list(all, mall, sall, chall)
	    self.roomur.list(all, mall, sall, chall)

    def energyold(self, e):
        "Computed the energy of the ...."
	e += self.penalty_width(self.xymm[2]-self.xymm[0])
	e += self.penalty_width(self.xymm[3]-self.xymm[1])
	if self.isleaf(): return e
	e = self.roomll.energy(e)
	e = self.roomur.energy(e)
	return e

    def energy(self):
        "Computed the energy of the ...."
	e  = self.penalty_width(self.xymm[2]-self.xymm[0])
	e += self.penalty_width(self.xymm[3]-self.xymm[1])
	return e
	
    col = "red yellow green blue cyan magenta white".split()

    def plot(self, ch, icol):
        "Plots the room into a ThanChart."
	xa, ya = self.xymm[:2]
	xb, yb = self.xymm[2:]
	xx = (xa, xb, xb, xa, xa)
	yy = (ya, ya, yb, yb, ya)
	ch.curveAdd(xx, yy, color=self.col[icol%len(self.col)])
	if not self.isleaf(): self.plotSplit(ch, icol+1)

    def plotSplit(self, ch, icol):
        "Plots the split line of the room."
	if self.isplit == 0:
	    xx = (self.csplit,  self.csplit)
	    yy = (self.xymm[1], self.xymm[3])
	else:
	    xx = (self.xymm[0], self.xymm[2])
	    yy = (self.csplit,  self.csplit)
	ch.curveAdd(xx, yy, color=self.col[icol%len(self.col)])
	if not self.roomll.isleaf(): self.roomll.plotSplit(ch, icol+1); icol += 1
	if not self.roomur.isleaf(): self.roomur.plotSplit(ch, icol+1)

    @staticmethod
    def penalty_width(b):
        "Computes the penalty due to small or big width."
	assert b>0, "b=%f !" % b   	
        if b < 2.5:
	    e = 3-2.5 + (2.5-b)*10
	elif b < 3.0:
	    e = 3-b
	elif b <= 4:
	    e = 0
	elif b < 5:
	    e = 5-b
	else:
	    e = 5-4 + (b-5)*10
	return e


class RoomConfiguration:
    "A room hierarchy amenable to simulated annealling."
    
    def __init__(self, b, h):
        "Initial configuration is a single room."
        self.root = Room((0.0,0.0,b,h))
	self.per = 0.2                      # Do not split nearer to 20% from sides
	self.r = random.Random()
	self.typeNodes()

    def energyStateold(self):
        "Computes the energy of the configuration."	
        return self.efact * self.root.energy(0.0) / len(self.all)

    def energyState(self):
        "Computes the energy of the configuration."
	e = 0.0	
        for room in self.sable:
	    e += room.energy()
	if len(self.sable) > 5: e += (len(self.sable)-5)*200
	return self.efact * e / len(self.sable)
	
    def changeState(self):
        "Changes the configuration a little."
	self.saveOldState()
	p = self.r.random()
	if p < 0.8 and self.chable:
#	    print "change"
	    room = self.r.choice(self.chable)
	    i = room.isplit
	    dc = (room.xymm[i+2] - room.xymm[i])*self.per
	    room.csplit = self.r.uniform(room.xymm[i]+dc, room.xymm[i+2]-dc)
	    room.roomll.xymm[i+2] = room.csplit
	    room.roomur.xymm[i]   = room.csplit
	elif p < 0.9 or not self.chable:   # If none is mergeable then split
#	    print "split"
	    room = self.r.choice(self.sable)
	    dx = room.xymm[2]-room.xymm[0]
	    dy = room.xymm[3]-room.xymm[1]
	    px = 1.0/(1.0+dy/dx)
	    if self.r.random() < px: i = 0
	    else:                    i = 1
	    dc = (room.xymm[i+2] - room.xymm[i])*self.per
	    c = self.r.uniform(room.xymm[i]+dc, room.xymm[i+2]-dc)
	    room.split(i, c)
	    self.typeNodes()
	else:
#	    print "merge"
	    room = self.r.choice(self.chable)
	    room.merge()
	    self.typeNodes()
	return len(self.all)       # Dimensionality of the configuration

    def saveOldState(self):
        "Save current configuration for future unChangeState()."
	self.rootOld = copy.deepcopy(self.root)

    def unChangeState(self):
        "Restores previously saved state."
#	print "unchange"
	self.root = self.rootOld
	self.typeNodes()
	return len(self.all)       # Dimensionality of the configuration
	
    def saveMinState(self):
        "Save current configuration as the best so far."
	self.rootMin = copy.deepcopy(self.root)

    def restoreMinState(self):
        "Restores previously saved best configuration."
	self.root = self.rootMin
	self.typeNodes()
	return len(self.all)       # Dimensionality of the configuration

    def initState(self):
      "Initialize random changes and calibrate energy."
      self.efact = 1.0
      ndim = 5                       # Assume 5 rooms in house	  
      tries = ndim*10               
      for i in xrange(tries):        # Randomise a bit
      	  j = self.changeState()
      e1 = self.energyState()
      de = 0.0
      ndim = len(self.all)
      tries = ndim*10               
      for i in xrange(tries):
          j = self.changeState()
          e2 = self.energyState()
          de += abs(e2-e1)
          e1 = e2
      de /= tries
      self.efact = 100.0 / de             # Normalise delta energy to 100: efact*de = 100
      prg(' =%.3f' % e1)
      prg("   =%.3f" % de)
      print "number of rooms:", len(self.all)
      print "efact=", self.efact
      self.plot("First approximation")
      return len(self.all)

    def typeNodes(self):
        "Finds a list of splittable, a list of mergeable nodes and a list of all."
	self.all = []
	self.mable = []
	self.sable = []
	self.chable = []
	self.root.list(self.all, self.mable, self.sable, self.chable)

    def plot(self, tit):
        from p_gchart import ThanChart, vis
        ch = ThanChart(title=tit)
	self.root.plot(ch, 0)
	vis(ch)

def test():
    "Test-builds a random room hierarchy."
    from p_gchart import ThanChart, vis
    r = random.Random()
    root = Room((0,0,20,10))
    global col
    col = "red yellow green blue cyan magenta white".split()
    per = 0.2
    for itry in xrange(5):
        all = []; mall = []; sall = []; chall = []
        root.list(all, mall, sall, chall)
	room = r.choice(sall)
	dx = room.xymm[2]-room.xymm[0]
	dy = room.xymm[3]-room.xymm[1]
	px = 1.0/(1.0+dy/dx)
	if r.random() < px: i = 0
	else:               i = 1
#        i = r.randint(0, 1)
	dc = (room.xymm[i+2] - room.xymm[i])*per
	c = r.uniform(room.xymm[i]+dc, room.xymm[i+2]-dc)
	room.split(i, c)
    print "energy=", root.energy(0.0)
    ch = ThanChart()
    root.plot(ch, 0)
    vis(ch)

if __name__ == "__main__": test()
