module modthanroomlist
use modparam
use modroomtype
implicit none
private
public :: ThanRoomList, init, free, append, get, clear, llen

type ThanNode
    type(Room), pointer :: data
    type(ThanNode), pointer :: next
end type thanNode

type ThanRoomList
    private
    integer(KIL) :: inow, len
    type(ThanNode), pointer :: head, z, now
end type ThanRoomList

interface init
    module procedure thanListInit
end interface init

interface free
    module procedure thanListFree
end interface free

interface append
!    module procedure thanListAppendpointer
    module procedure thanListAppendtarget
end interface append

interface get
    module procedure thanListGet
end interface get

interface set
    module procedure thanListSet
end interface set

interface clear
    module procedure thanListClear
end interface clear

interface llen
    module procedure thanListLen
end interface llen

contains

subroutine thanListInit(self)
!   "Make an empty list."
    type(ThanRoomList), intent(inout) :: self
    allocate (self%head, self%z)
    self%head%next => self%z
    self%z%next => self%z
    nullify (self%head%data, self%z%data)
    self%len = 0
    self%inow = 0
    self%now => self%head
end subroutine thanListInit

subroutine thanListFree(self)
!   "Delete the list."
    type(ThanRoomList), intent(inout) :: self
    integer(KIL) :: i
    type(ThanNode), pointer :: temp, next
    next => self%head%next
    do i=1, self%len
        temp => next
        next => temp%next
        deallocate (temp)
    end do
    deallocate (self%head, self%z)
    nullify (self%head, self%z, self%now)
    self%inow = -1                             !ThanList has been deleted
end subroutine thanListFree

subroutine thanListAppendpointer(self, a)
!   "Append an element to the list."
    type(ThanRoomList), intent(inout) :: self
    type(Room), pointer :: a
    type(ThanNode), pointer :: last
    last => self%z
    last%data => a
    allocate (self%z)
    nullify (self%z%data)
    last%next => self%z
    self%z%next => self%z
    self%len = self%len + 1
end subroutine thanListAppendpointer

subroutine thanListAppendtarget(self, a)
!   "Append an element to the list."
    type(ThanRoomList), intent(inout) :: self
    type(Room), target, intent(in) :: a
    type(ThanNode), pointer :: last
    last => self%z
    last%data => a
    allocate (self%z)
    nullify (self%z%data)
    last%next => self%z
    self%z%next => self%z
    self%len = self%len + 1
end subroutine thanListAppendtarget

function thanListGet(self, i)
!   "Get a list element."
    type(Room), pointer :: thanListGet 
    type(ThanRoomList), intent(inout) :: self
    integer(KIL), intent(in) :: i
    call thanListSetcur(self, i)
    thanListGet => self%now%data
end function thanListGet

subroutine thanListSet(self, i, a)
!   "Set a list element."
    type(ThanRoomList), intent(inout) :: self
    type(Room), pointer :: a
    integer(KIL), intent(in) :: i
    call thanListSetcur(self, i)
    self%now%data => a
end subroutine thanListSet

subroutine thanListSetcur(self, i)
!   "Sets the current element to i."
    type(ThanRoomList), intent(inout) :: self
    integer(KIL), intent(in) :: i
    integer(KIL) :: j
    if (i > self%len .or. i < 1) stop "ThanList:setcur: IndexError"
    if (i >= self%inow) then
        do j=self%inow+1, i
            self%inow = self%inow + 1
	    self%now => self%now%next
	end do
    else
        self%now => self%head
        self%inow = 0
        do j=self%inow+1, i
            self%inow = self%inow + 1
	    self%now => self%now%next
	end do
    end if
end subroutine thanListSetcur

subroutine thanListClear (self)
!   "Clears all the elements the list; does not delete the data of the list."
    type(ThanRoomList), intent(inout) :: self
    integer(KIL) :: i
    type(ThanNode), pointer :: temp, next
    next => self%head%next
    do i=1, self%len
        temp => next
	next => temp%next
        deallocate (temp)
    end do
    self%head%next => self%z
    self%len = 0
    self%inow = 0
    self%now => self%head
end subroutine thanListClear

integer(KIL) function thanListLen(self)
!   "Returns the length of the list."
    type(ThanRoomList), intent(in) :: self
    thanListLen = self%len
end function thanListLen

end module modthanroomlist
