# -*- coding: iso-8859-7 -*-
import random
from math import exp
from p_ggen import prg


class SimulatedAnnealing:
  "The class implements the simulated annealing method."
  
  def __init__(self, prt=prg):
      """Initialize simulated annealing procedure.
      
      These values are typical. They will be overwritten by anneal()."""
      self.prt = prt
      self.nTsteps  = 30   # max temperature steps
      self.nChan    = 100  # max number of state changes tried at any temperature
      self.nSch     = 10   # max number of succesful state changes
 
      self.iSch     = 0    # number of succesful state changes (positive or negative)
      self.iSchNeg  = 0    # number of succesful energy-negative state changes

      self.nDimState = 20  # The dimension of the vector space which each state
                           #  belongs to. Note that it may be not constant.

      self.t      = 100.0  # temperature
      self.tFactr = 0.70   # Annealing schedule: t is reduced by this factor on each step
      self.e1     = 0.0    # energy before and ..
      self.e2     = 0.0    # after a change
      self.emin   = 0.0    # minimum energy found
      self.r = random.Random()
      

  def anneal(self, obj):
      "Does the annealing process."
#      include 'anneal.inc'
#      integer*4 iTstep
#      real*8 eprev
#      character*78 dl
#     Initial average Delta Energy (just run the algorithm for ndimstate times, calculate 
#     delta energy and take the average) should be normalised to about 100

      self.nTsteps = 50       # max temperature steps
      self.tFactr  = 0.70     # Annealing schedule: t is reduced by this factor on each step
      self.t = 100.0          # initial temperature

#-----Initial sharing

      self.nDimState = obj.initState()
      self.e1 = obj.energyState()
      self.emin = self.e1
      eprev = self.e1
      obj.saveMinState()
      self.prt(' ')
      self.prt('')
      dl = "%-3s  %-8s  %-6s  %-6s  %-6s  %-8s  %-8s" % ('/', '',
           ' ', '', '', ' .', 'max .')
      self.prt(dl)

#-----Temperature loop-------------------------------------------

      form = "%3d%10.5f%8d%8d%8d%10.3f%10.3f"
      for iTstep in xrange(self.nTsteps):
          self.annealTrials(obj)
          self.prt(form % (iTstep, self.t, self.iSch, self.iSchNeg, self.iSch-self.iSchNeg,
          eprev-self.e1, eprev-self.emin))
          print "T=%10.5f  rooms=%d" % (self.t, len(obj.sable))
          if self.iSch == 0: break     # If no succesful changes, end.
          self.t *= self.tFactr
          eprev = self.e1	

      self.nDimState = obj.restoreMinState()

#============================================================================

  def annealTrials(self, obj):
      "Annealling fro 1 temperature."
#      include 'anneal.inc'
#      real*8 energyState                          ! FUNCTIONS
#      logical*2 metrop                            ! FUNCTIONS
#      external energyState, metrop
#      integer*2 nDsMax
#      integer*4 iChan

#-----Trial sharings' loop

      self.nChan  = 200 * self.nDimState  # max number of state changes tried at any temperature
      self.nSch   =  20 * self.nDimState  # max number of succesful bananas changes
      nDsMax = self.nDimState        # maximum dimension at this temperature

      self.iSch    = 0     # number of succesful state changes (positive or negative)
      self.iSchNeg = 0     # number of succesful energy-negative state changes
      iChan = 0

      while True:
          iChan += 1
#	  print "iChan=", iChan
          if iChan > self.nChan: return
	
          self.nDimState = obj.changeState()
          self.e2 = obj.energyState()
#	  print "e1,e2=", self.e1, self.e2

          if self.metrop(self.e2-self.e1, self.t):
              if self.e2 < self.emin:
                  self.emin = self.e2
                  obj.saveMinState()
              if self.nDimState > nDsMax:
                  self.nChan = 100 * self.nDimState
                  self.nSch  =  10 * self.nDimState
                  nDsMax = self.nDimState

              if self.e2-self.e1 <  0.0: self.iSchNeg = self.iSchNeg + 1
              if self.e2-self.e1 != 0.0: self.iSch = self.iSch + 1
              self.e1 = self.e2
              if self.iSch > self.nSch: return
          else:
              self.nDimState = obj.unChangeState()

#===========================================================================

  def metrop(self, de, t):
      "Metropolis algorithm."
#      print "metrop de, de/t=", de, de/t
      if de < 0.0: return True
      x = self.r.random()
      xm = exp(-de/t)
#      print "metrop xm,x=", xm, x
      return (x < xm)
