#!/usr/bin/python
# -*- coding: iso-8859-7 -*-
import random, copy
from functools import partial
from p_ggen import prg

class Room:
    "A class which represents a room, posible divided."

    def __init__(self, xymm):
        "The xymm are the coordinate of lower left followe by the coordinate  of upper right."
        self.xymm = list(xymm)
        self.isplit = None         # Room is not (yet) split
        self.csplit = None
        self.roomll = self.roomur = None # Children

    def split(self, isplit, csplit):
        "Split the room to 2 children."
	assert self.isleaf(), "room is already split!"
	assert self.xymm[isplit] < csplit < self.xymm[isplit+2], "Split line outside of the room"
	self.isplit = isplit
	self.csplit = csplit
	xymm = list(self.xymm)
	xymm[isplit+2] = csplit
	self.roomll = Room(xymm)       
	xymm = list(self.xymm)
	xymm[isplit] = csplit
	self.roomur = Room(xymm)       

    def mergeold(self):
        "Merges the room's children."
	assert self.roomll.isleaf() and self.roomur.isleaf(), "Children are not single rooms"
	self.isplit = self.csplit = self.roomll = self.roomur = None

    def merge(self):
        "Merges all the children/grandchildren of the room."
	self.isplit = self.csplit = self.roomll = self.roomur = None

    def isleaf(self):
        "Checks if the room is a leaf node (it has not been split)."
	return self.isplit == None

    def list(self, all, mall, sall, chall):
        "Returns a list of self and children."
	all.append(self)
	if self.isleaf():
	    sall.append(self)
	else:
	    if self.roomll.isplit == None and \
	       self.roomur.isplit == None: chall.append(self)
	    mall.append(self)
	    self.roomll.list(all, mall, sall, chall)
	    self.roomur.list(all, mall, sall, chall)

    def energyold(self, e):
        "Computed the energy of the ...."
	e += self.penalty_width(self.xymm[2]-self.xymm[0])
	e += self.penalty_width(self.xymm[3]-self.xymm[1])
	if self.isleaf(): return e
	e = self.roomll.energy(e)
	e = self.roomur.energy(e)
	return e

    def energy(self):
        "Computed the energy of the ...."
	e  = self.penalty_width(self.xymm[2]-self.xymm[0])
	e += self.penalty_width(self.xymm[3]-self.xymm[1])
	return e
	


    col = "red yellow green blue cyan magenta white".split()
    def plot(self, ch, icol):
        "Plots the room into a ThanChart."
	xa, ya = self.xymm[:2]
	xb, yb = self.xymm[2:]
	xx = (xa, xb, xb, xa, xa)
	yy = (ya, ya, yb, yb, ya)
	ch.curveAdd(xx, yy, color=self.col[icol%len(self.col)])
	if not self.isleaf(): self.plotSplit(ch, icol+1)


    def plotSplit(self, ch, icol):
        "Plots the split line of the room."
	if self.isplit == 0:
	    xx = (self.csplit,  self.csplit)
	    yy = (self.xymm[1], self.xymm[3])
	else:
	    xx = (self.xymm[0], self.xymm[2])
	    yy = (self.csplit,  self.csplit)
	ch.curveAdd(xx, yy, color=self.col[icol%len(self.col)])
	if not self.roomll.isleaf(): self.roomll.plotSplit(ch, icol+1); icol += 1
	if not self.roomur.isleaf(): self.roomur.plotSplit(ch, icol+1)


    def plot2(self, proj, icol):
        "Plots the room into an active ThanCad drawing."
        from thansupport import thanToplayerCurrent
        from thandr import ThanLine
        col1 = self.col[icol%len(self.col)]
        thanToplayerCurrent(proj, "floorplan/%s"%col1, moncolor=col1)
        cn = partial(cnp, prototype=proj[1].thanVar["elevation"])
        xa, ya = self.xymm[0], self.xymm[1]
        xb, yb = self.xymm[2], self.xymm[3]
        cc = [cn(xa, ya), cn(xb, ya), cn(xb, yb), cn(xa, yb), cn(xa, ya)]
        e = ThanLine()
        e.thanSet(cc)
        proj[1].thanElementAdd(e)
        e.thanTkDraw(proj[2].than)
        if not self.isleaf(): self.plotSplit2(proj, icol+1)


    def plotSplit2(self, proj, icol):
        "Plots the split line of the room into an active ThanCad drawing."
        from thansupport import thanToplayerCurrent
        from thandr import ThanLine
        col1 = self.col[icol%len(self.col)]
        thanToplayerCurrent(proj, "floorplan/%s"%col1, moncolor=col1)
        cn = partial(cnp, prototype=proj[1].thanVar["elevation"])
        if self.isplit == 0:
            cc = [cn(self.csplit, self.xymm[1]), cn(self.csplit, self.xymm[3])]
        else:
            cc = [cn(self.xymm[0], self.csplit),  cn(self.xymm[2], self.csplit)]
        e = ThanLine()
        e.thanSet(cc)
        proj[1].thanElementAdd(e)
        e.thanTkDraw(proj[2].than)
        if not self.roomll.isleaf(): self.roomll.plotSplit2(proj, icol+1); icol += 1
        if not self.roomur.isleaf(): self.roomur.plotSplit2(proj, icol+1)


    @staticmethod
    def penalty_width(b):
        "Computes the penalty due to small or big width."
	assert b>0, "b=%f !" % b   	
        if b < 2.5:
	    e = 3-2.5 + (2.5-b)*10
	elif b < 3.0:
	    e = 3-b
	elif b <= 4:
	    e = 0
	elif b < 5:
	    e = 5-b
	else:
	    e = 5-4 + (b-5)*10
	return e


class RoomConfiguration:
    "A room hierarchy amenable to simulated annealling."
    
    def __init__(self, xor, yor, b, h, nrooms, prt=prg):
        "Initial configuration is a single room."
        self.prt = prt
        self.root = Room((xor,yor,xor+b,yor+h))
        self.nrooms = nrooms
        self.per = 0.2                      # Do not split nearer to 20% from sides
        self.r = random.Random()
        self.typeNodes()

    def energyStateold(self):
        "Computes the energy of the configuration."
        return self.efact * self.root.energy(0.0) / len(self.all)

    def energyState(self):
        "Computes the energy of the configuration."
        e = 0.0
        for room in self.sable:
            e += room.energy()
        if len(self.sable) > self.nrooms: e += (len(self.sable)-self.nrooms)*200
        return self.efact * e / len(self.sable)

    def changeState(self, fill=False):
        "Changes the configuration a little."
	self.saveOldState()
	p = self.r.random()
	if p < 0.4 and self.chable:     # In case that only the root room exists
#	    print "change"
	    room = self.r.choice(self.chable)
	    i = room.isplit
	    dc = (room.xymm[i+2] - room.xymm[i])*self.per
	    room.csplit = self.r.uniform(room.xymm[i]+dc, room.xymm[i+2]-dc)
	    room.roomll.xymm[i+2] = room.csplit
	    room.roomur.xymm[i]   = room.csplit
	elif p < 0.7 or not self.chable or (fill and len(self.sable)<self.nrooms):# and len(self.sable) <=self.nrooms:   # If none is mergeable then split
#	    print "split"
	    room = self.r.choice(self.sable)
	    dx = room.xymm[2]-room.xymm[0]
	    dy = room.xymm[3]-room.xymm[1]
	    px = 1.0/(1.0+dy/dx)
	    if self.r.random() < px: i = 0
	    else:                    i = 1
	    dc = (room.xymm[i+2] - room.xymm[i])*self.per
	    c = self.r.uniform(room.xymm[i]+dc, room.xymm[i+2]-dc)
	    room.split(i, c)
	    self.typeNodes()
	else:
#	    print "merge"
	    room = self.r.choice(self.chable)
	    room.merge()
	    self.typeNodes()
	return len(self.all)       # Dimensionality of the configuration

    def saveOldState(self):
        "Save current configuration for future unChangeState()."
	self.rootOld = copy.deepcopy(self.root)

    def unChangeState(self):
        "Restores previously saved state."
#	print "unchange"
	self.root = self.rootOld
	self.typeNodes()
	return len(self.all)       # Dimensionality of the configuration
	
    def saveMinState(self):
        "Save current configuration as the best so far."
	self.rootMin = copy.deepcopy(self.root)

    def restoreMinState(self):
        "Restores previously saved best configuration."
	self.root = self.rootMin
	self.typeNodes()
	return len(self.all)       # Dimensionality of the configuration

    def initState(self):
      "Initialize random changes and calibrate energy."
      self.efact = 1.0
      ndim = self.nrooms             # Assume (nrooms) rooms in house
      tries = ndim*10
      for i in xrange(tries):        # Randomise a bit
          j = self.changeState(fill=True)    # (almost) ensure that number of rooms is about nrooms
      e1 = self.energyState()
      de = 0.0
      ndim = len(self.all)
      tries = ndim*10
      for i in xrange(tries):
          j = self.changeState()
          e2 = self.energyState()
          de += abs(e2-e1)
          e1 = e2
      de /= tries
      self.efact = 100.0 / de             # Normalise delta energy to 100: efact*de = 100
      self.prt(' =%.3f' % e1)
      self.prt("   =%.3f" % de)
      print "number of rooms:", len(self.sable)
      print "efact=", self.efact
#      self.plot("First approximation")
      return len(self.all)

    def typeNodes(self):
        "Finds a list of splittable, a list of mergeable nodes and a list of all."
	self.all = []
	self.mable = []
	self.sable = []
	self.chable = []
	self.root.list(self.all, self.mable, self.sable, self.chable)


    def plot(self, tit):
        from p_gchart import ThanChart, vis
        ch = ThanChart(title=tit)
        self.root.plot(ch, 0)
        vis(ch)


    def plot2(self, proj):
        self.root.plot2(proj, 0)


def cnp(xa, ya, prototype):
    "Return the coordinates as list like prototype."
    ca = list(prototype)
    ca[:2] = xa, ya
    return ca


def test():
    "Test-builds a random room hierarchy."
    from p_gchart import ThanChart, vis
    r = random.Random()
    root = Room((0,0,20,10))
    global col
    col = "red yellow green blue cyan magenta white".split()
    per = 0.2
    for itry in xrange(5):
        all = []; mall = []; sall = []; chall = []
        root.list(all, mall, sall, chall)
	room = r.choice(sall)
	dx = room.xymm[2]-room.xymm[0]
	dy = room.xymm[3]-room.xymm[1]
	px = 1.0/(1.0+dy/dx)
	if r.random() < px: i = 0
	else:               i = 1
#        i = r.randint(0, 1)
	dc = (room.xymm[i+2] - room.xymm[i])*per
	c = r.uniform(room.xymm[i]+dc, room.xymm[i+2]-dc)
	room.split(i, c)
    print "energy=", root.energy(0.0)
    ch = ThanChart()
    root.plot(ch, 0)
    vis(ch)

if __name__ == "__main__": test()
