# -*- coding: iso-8859-7 -*-
##############################################################################
# ThanCad 0.2.2 "Urban SAR": 2dimensional CAD with raster support for engineers.
# 
# Copyright (c) 2001-2013 Thanasis Stamos,  January 16, 2013
# URL:     http://thancad.sourceforge.net
# e-mail:  cyberthanasis@excite.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (www.gnu.org/licenses/gpl.html).
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################

"""\
ThanCad 0.2.2 "Urban SAR": 2dimensional CAD with raster support for engineers.

Package which creates a bioclimatic city plan.
This module defines circular urban plan system.
"""
from math import cos, sin, pi
import itertools
from PIL import Image, ImageDraw
import p_ggen, p_gimdxf, p_gdxf, p_gchart, p_gmath, p_gtri, p_ggeom
import roadut, roadem, hippo
import geomcirc


class CircularUrban(hippo.HippoUrban):
    "An object which contains a circular urban plan system: a grid of roads with nonuniform grid spaces."

    def testdxf(self, state, boik1):
        "Plot the current state as a test."
        import p_ggeom, p_gdxf
        dxf = p_gdxf.ThanDxfPlot()
        dxf.thanDxfPlots()
        hull = self.hull
        x = [c[0] for c in self.hull[:4]]
        y = [c[1] for c in self.hull[:4]]
        m = p_ggeom.MesaQuad(x[0], y[0], x[1], y[1], x[2], y[2], x[3], y[3])
        x.append(x[0])
        y.append(y[0])
        dxf.thanDxfPlotPolyline(x, y)

        r = 0.0
        rs = []
        plines = {}
        for h in state.hoik:
            r += h
            rs.append(r)
            plines[r] = geomcirc.arcs(state.cen, r, lambda c: m.mesa(c[0], c[1]), stepdist=20.0, stepmin=1.0)
            for cs in plines[r]:
                x = [c[0] for c in cs]
                y = [c[1] for c in cs]
                dxf.thanDxfPlotPolyline(x, y)
        ths = []
        th = 0.0
        for dphi1 in state.dphi:
           th += dphi1
           ths.append(th)
        dxf.thanDxfSetLayer("RAYS")
        dxf.thanDxfPlotPoint(state.cen[0], state.cen[1])
#        radial1(cen, dxf, ths, rs, plines)
        geomcirc.radial2(state.cen, dxf, ths, rs, plines, boik1)
        dxf.thanDxfPlot(0,0,999)
