# -*- coding: iso-8859-7 -*-

##############################################################################
# ThanCad 0.1.2 "Decade": 2dimensional CAD with raster support for engineers.
# 
# Copyright (c) 2001-2011 Thanasis Stamos,  August 20, 2011
# URL:     http://thancad.sourceforge.net
# e-mail:  cyberthanasis@excite.com
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details (www.gnu.org/licenses/gpl.html).
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################

"""\
ThanCad 0.1.2 "Decade": 2dimensional CAD with raster support for engineers.

This module defines the base for the dialogs related to arhitectural algorithms.
"""

from types import *
import Tkinter, tkFont
from thanvar import thanicon, Canc
from thantrans import Tarch as T
import p_gtkuti, p_gtkwid, p_ggen
class ThanArchCom(p_gtkuti.ThanDialog):
    "An object which provides for functionality common to ICP algorithms."

    def __init__(self, *args, **kw):
        p_gtkuti.ThanDialog.__init__(self, *args, **kw)


    def thanValsDefArch(self, v):
        "Build default values."
        v.entWidth = 9.0
        v.entHeight = 7.0


    def thanValsReadArch(self, c, tv, v):
        "Reads the values from a file with specific suffix."
        sec = "FLOOR PLAN CONSTRAINTS"
        for key in "entWidth entHeight".split():
            tit, val = tv[key]
            try:
                test = c.get(sec, tit)
                test = val.thanValidate(test)
                if test != None: setattr(v, key, test)
            except:
                pass


    def thanValsWriteArch(self, c, tv, v):
        "Write the values to a file with specific suffix."
        sec = "FLOOR PLAN CONSTRAINTS"
        if not c.has_section(sec): c.add_section(sec)
        for key in "entWidth entHeight".split():
            tit, val = tv[key]
            c.set(sec, tit, str(getattr(v, key)))


    def fraLogo(self, win, ir, theme=T["Automated Floor Plan Design Algorithms"], year=2009):
        "Display the logo."
        fra = Tkinter.Frame(win, bd=3, relief=Tkinter.RIDGE)
        fra.grid(row=ir, column=0, pady=5, sticky="we")
        self.fo1 = tkFont.Font(family="Liberation Serif", weight="bold", size=18)

        lab = Tkinter.Label(fra, image=thanicon.get("ntua3"))
        lab.grid(row=0, column=0, rowspan=3)

        wid = Tkinter.Frame(fra)
        wid.grid(row=0, column=1, rowspan=3, sticky="we")

        lab = Tkinter.Label(fra, text=theme, font=self.fo1, fg="blue")
        lab.grid(row=0, column=2, sticky="w")
        lab = Tkinter.Label(fra, text=T["Thanasis Stamos, Research/Teaching Personnel"], font=self.fo1, fg="blue")
        lab.grid(row=1, column=2, sticky="w")
        lab = Tkinter.Label(fra, text=T["School of Civil Engineering, NTUA, "]+str(year), font=self.fo1, fg="blue")
        lab.grid(row=2, column=2, sticky="w")

        wid = Tkinter.Frame(fra)
        wid.grid(row=0, column=3, rowspan=3, sticky="we")

        lab = Tkinter.Label(fra, image=thanicon.get("than"))
        lab.grid(row=0, column=4, rowspan=3)
        fra.columnconfigure(1, weight=1)
        fra.columnconfigure(3, weight=1)


    def fraGeom(self, win, ir):
        "Select the geometry of the floor plan."
        fra = Tkinter.Frame(win, bd=3, relief=Tkinter.RIDGE)
        fra.grid(row=ir, column=0, pady=5, sticky="we")

        lab = Tkinter.Label(fra, fg=self.colfra, text="%d."%(ir,))
        lab.grid(row=0, column=0)
        lab = Tkinter.Label(fra, anchor="w", fg=self.colfra, text=T["FLOOR PLAN CONSTRAINTS:"])
        lab.grid(row=0, column=1, sticky="w", columnspan=4)

        key = "entWidth"
        tit = u"Floor plan width (m)"
        val = p_gtkwid.ThanValFloat(0.10, 1000.0)
        lab = Tkinter.Label(fra, text=T[tit])
        lab.grid(row=1, column=1, sticky="e")
        wid = p_gtkwid.ThanEntry(fra, width=5)
        wid.grid(row=1, column=2, sticky="we")
        self.thanWids.append((key, tit, wid, val))

        key = "entHeight"
        tit = "Floor plan height (m)"
        val = p_gtkwid.ThanValFloat(0.10, 1000.0)
        lab = Tkinter.Label(fra, text=T[tit])
        lab.grid(row=2, column=1, sticky="e")
        wid = p_gtkwid.ThanEntry(fra, width=5)
        wid.grid(row=2, column=2, sticky="we")
        self.thanWids.append((key, tit, wid, val))

        wid = Tkinter.Frame(fra)
        wid.grid(row=1, column=3, padx=30)

        key = "entMinRooms"
        tit = "Min number of rooms"
        val = p_gtkwid.ThanValInt(1, 1000)
        lab = Tkinter.Label(fra, text=T[tit])
        lab.grid(row=1, column=4, sticky="e")
        wid = p_gtkwid.ThanEntry(fra, width=5)
        wid.grid(row=1, column=5, sticky="we")
        self.thanWids.append((key, tit, wid, val))

        key = "entMaxRooms"
        tit = "Max number of rooms"
        val = p_gtkwid.ThanValInt(1, 1000)
        lab = Tkinter.Label(fra, text=T[tit])
        lab.grid(row=2, column=4, sticky="e")
        wid = p_gtkwid.ThanEntry(fra, width=5)
        wid.grid(row=2, column=5, sticky="we")
        self.thanWids.append((key, tit, wid, val))

        fra.columnconfigure(3, weight=1)


    def thanSet(self, vs):
        "Set new values to the widgets."
        stat = Tkinter.NORMAL
        for (key,tit,wid,vld) in self.thanWids:
            v = getattr(vs, key)
            if type(v) == FloatType or type(v) == IntType: v = str(v)
            wid.config(state=stat) # All widgets must be enabled..
            wid.thanSet(v)         # ..to change their values


    def validate(self, strict=True):
        """Returns true if the value chosen by the user is valid.

        If strict == True, then if an error is found, an error message is displayed,
        self.result is unchanged, and False is returned to the caller.
        If strict == True, and no errors are found, self.result is updated with
        the new values. True is returned to the caller.
        If strict == False, then if an error is found, a default value is used
        instead of the wrong one, self.results is set with the new values,
        and False is returned to the caller.
        If strict == False, and no errors are found, then, self.results is set
        with the new values, and True is returned to the caller.
        """
        ret = True
        vs = p_ggen.Struct()
        for key,tit,wid,vld in self.thanWids:
            v1 = vld.thanValidate(wid.thanGet())
            if v1 == None:
                ret = False
                if strict:
                    tit = u'"%s":\n%s' % (tit, vld.thanGetErr())
                    p_gtkuti.thanGudModalMessage(self, tit, T["Error in data"])
                    self.initial_focus = wid
                    return ret
                else:
                    v1 = getattr(self.thanValsInit, key)
            setattr(vs, key, v1)
        return ret, vs


    def apply(self):
        "Last settings - after successful validation and after the window has been closed."
        self.thanValsWrite(self.result)


    def cancel(self, *args):
        "Ask before cancel."
        ret = self.validate(strict=False)     # If anything is wrong, then let it be
        if self.result.__dict__ != self.thanValsSaved.__dict__:
            print "result=", self.result.__dict__
            print "saved=", self.thanValsSaved.__dict__
            a = p_gtkuti.thanGudAskOkCancel(self, T["Data modified, OK to cancel?"], T["Warning"])
            if not a: return        # Cancel was stopped
        p_gtkuti.ThanDialog.cancel(self, *args)


    def destroy(self, *args):
        "Deletes references to widgets, so that it breaks circular references."
        for (key,tit,wid,vld) in self.thanWids:
            delattr(self, key)
#        del self.thanLabGps, self.thanLabRel,
        del self.thanProj, self.thanWids, self.thanValsInit, self.thanValsSaved
        p_gtkuti.ThanDialog.destroy(self)


    def __del__(self):
        print "ThanArchcom ThanDialog", self, "dies.."
